@@ -1,7 +1,7 @@
 /**
  *
  * getdns dict management functions, note that the internal storage is
- * accomplished via an getdns_rbtree_t
+ * accomplished via an _getdns_rbtree_t
  *
  * Interfaces originally taken from the getdns API description pseudo implementation.
  *
@@ -56,27 +56,27 @@
  * @return pointer to dictionary item, caller must not free storage associated with item
  * @return NULL if additnotfnd == FALSE and key is not in dictionary
  */
-struct getdns_dict_item *
+static struct getdns_dict_item *
 getdns_dict_find(const struct getdns_dict *dict, const char *key)
 {
 	return (struct getdns_dict_item *)
-		   getdns_rbtree_search((getdns_rbtree_t *)&(dict->root), key);
+		   _getdns_rbtree_search((_getdns_rbtree_t *)&(dict->root), key);
 }				/* getdns_dict_find */
 
-struct getdns_dict_item *
+static struct getdns_dict_item *
 getdns_dict_find_and_add(struct getdns_dict *dict, const char *key)
 {
 	struct getdns_dict_item *item;
 
 	item = (struct getdns_dict_item *)
-		   getdns_rbtree_search(&(dict->root), key);
+		   _getdns_rbtree_search(&(dict->root), key);
 
 	if (!item) {
 		/* add a node */
 		item = GETDNS_MALLOC(dict->mf, struct getdns_dict_item);
-		item->node.key = getdns_strdup(&dict->mf, key);
+		item->node.key = _getdns_strdup(&dict->mf, key);
 		item->data.n = 0;
-		getdns_rbtree_insert(&(dict->root), (getdns_rbnode_t *) item);
+		_getdns_rbtree_insert(&(dict->root), (_getdns_rbnode_t *) item);
 	}
 	return item;
 }				/* getdns_dict_find_and_add */
@@ -85,12 +85,9 @@ getdns_dict_find_and_add(struct getdns_dict *dict, const char *key)
 /*---------------------------------------- getdns_dict_get_names
 */
 getdns_return_t
-getdns_dict_get_names(const struct getdns_dict * dict,
-	struct getdns_list ** answer)
+getdns_dict_get_names(const getdns_dict *dict, getdns_list **answer)
 {
 	struct getdns_dict_item *item;
-	size_t index;
-	struct getdns_bindata bindata;
 
 	if (!dict || !answer)
 		return GETDNS_RETURN_INVALID_PARAMETER;
@@ -102,12 +99,8 @@ getdns_dict_get_names(const struct getdns_dict * dict,
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 
 	RBTREE_FOR(item, struct getdns_dict_item *,
-		(getdns_rbtree_t *)&(dict->root)) {
-		if (getdns_list_add_item(*answer, &index) != GETDNS_RETURN_GOOD)
-			continue;
-		bindata.size = strlen(item->node.key) + 1;
-		bindata.data = (void *) item->node.key;
-		getdns_list_set_bindata(*answer, index, &bindata);
+		(_getdns_rbtree_t *)&(dict->root)) {
+		_getdns_list_append_string(*answer, item->node.key);
 	}
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_get_names */
@@ -241,7 +234,7 @@ getdns_dict_create_with_extended_memory_functions(
 	dict->mf.mf.ext.realloc = realloc;
 	dict->mf.mf.ext.free    = free;
 
-	getdns_rbtree_init(&(dict->root),
+	_getdns_rbtree_init(&(dict->root),
 		(int (*)(const void *, const void *)) strcmp);
 	return dict;
 }
@@ -278,7 +271,7 @@ getdns_dict_create()
 	return getdns_dict_create_with_context(NULL);
 }					/* getdns_dict_create */
 
-/*---------------------------------------- getdns_dict_copy */
+/*---------------------------------------- _getdns_dict_copy */
 /**
  * private function used to make a copy of a dict structure,
  * the caller is responsible * for freeing storage allocated to returned value
@@ -288,7 +281,7 @@ getdns_dict_create()
  * @return NULL on error (out of memory, invalid srcdict)
  */
 getdns_return_t
-getdns_dict_copy(const struct getdns_dict * srcdict,
+_getdns_dict_copy(const struct getdns_dict * srcdict,
 	struct getdns_dict ** dstdict)
 {
 	struct getdns_dict_item *item;
@@ -312,7 +305,7 @@ getdns_dict_copy(const struct getdns_dict * srcdict,
 
 	retval = GETDNS_RETURN_GOOD;
 	RBTREE_FOR(item, struct getdns_dict_item *,
-		(struct getdns_rbtree_t *)&(srcdict->root)) {
+		(struct _getdns_rbtree_t *)&(srcdict->root)) {
 		key = (char *) item->node.key;
 		switch (item->dtype) {
 		case t_bindata:
@@ -342,7 +335,7 @@ getdns_dict_copy(const struct getdns_dict * srcdict,
 		}
 	}
 	return GETDNS_RETURN_GOOD;
-}				/* getdns_dict_copy */
+}				/* _getdns_dict_copy */
 
 /*---------------------------------------- getdns_dict_item_free */
 /**
@@ -352,8 +345,8 @@ getdns_dict_copy(const struct getdns_dict * srcdict,
  *            to free the items
  * @return void
  */
-void
-getdns_dict_item_free(getdns_rbnode_t * node, void *arg)
+static void
+getdns_dict_item_free(_getdns_rbnode_t * node, void *arg)
 {
 	struct getdns_dict_item *item = (struct getdns_dict_item *) node;
 	struct getdns_dict *dict = (struct getdns_dict *)arg;
@@ -363,7 +356,7 @@ getdns_dict_item_free(getdns_rbnode_t * node, void *arg)
 
 	switch (item->dtype) {
 	case t_bindata:
-		getdns_bindata_destroy(&dict->mf, item->data.bindata);
+		_getdns_bindata_destroy(&dict->mf, item->data.bindata);
 		break;
 	case t_dict:
 		getdns_dict_destroy(item->data.dict);
@@ -384,7 +377,7 @@ void
 getdns_dict_destroy(struct getdns_dict *dict)
 {
 	if (!dict) return;
-	getdns_traverse_postorder(&(dict->root), getdns_dict_item_free, dict);
+	_getdns_traverse_postorder(&(dict->root), getdns_dict_item_free, dict);
 	GETDNS_FREE(dict->mf, dict);
 }				/* getdns_dict_destroy */
 
@@ -400,7 +393,7 @@ getdns_dict_set_dict(struct getdns_dict * dict, const char *name,
 	if (!dict || !name || !child_dict)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	retval = getdns_dict_copy(child_dict, &newdict);
+	retval = _getdns_dict_copy(child_dict, &newdict);
 	if (retval != GETDNS_RETURN_GOOD)
 		return retval;
 
@@ -426,7 +419,7 @@ getdns_dict_set_list(struct getdns_dict * dict, const char *name,
 	if (!dict || !name || !child_list)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	retval = getdns_list_copy(child_list, &newlist);
+	retval = _getdns_list_copy(child_list, &newlist);
 	if (retval != GETDNS_RETURN_GOOD)
 		return retval;
 
@@ -451,13 +444,13 @@ getdns_dict_set_bindata(struct getdns_dict * dict, const char *name,
 	if (!dict || !name || !child_bindata)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
-	newbindata = getdns_bindata_copy(&dict->mf, child_bindata);
+	newbindata = _getdns_bindata_copy(&dict->mf, child_bindata);
 	if (!newbindata)
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 
 	item = getdns_dict_find_and_add(dict, name);
 	if (!item) {
-		getdns_bindata_destroy(&dict->mf, newbindata);
+		_getdns_bindata_destroy(&dict->mf, newbindata);
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 	}
 	item->dtype = t_bindata;
@@ -465,6 +458,35 @@ getdns_dict_set_bindata(struct getdns_dict * dict, const char *name,
 	return GETDNS_RETURN_GOOD;
 }				/* getdns_dict_set_bindata */
 
+/*---------------------------------------- getdns_dict_set_bindata */
+getdns_return_t
+getdns_dict_util_set_string(getdns_dict *dict, char *name, const char *value)
+{
+	struct getdns_dict_item *item;
+	getdns_bindata          *newbindata;
+
+	if (!dict || !name || !value)
+		return GETDNS_RETURN_INVALID_PARAMETER;
+
+	if (!(newbindata = GETDNS_MALLOC(dict->mf, getdns_bindata)))
+		return GETDNS_RETURN_MEMORY_ERROR;
+
+	newbindata->size = strlen(value);
+	if (!(newbindata->data = (void *)_getdns_strdup(&dict->mf, value)))
+		goto error_free_bindata;
+
+	if ((item = getdns_dict_find_and_add(dict, name))) {
+
+		item->dtype = t_bindata;
+		item->data.bindata = newbindata;
+		return GETDNS_RETURN_GOOD;
+	}
+	GETDNS_FREE(dict->mf, newbindata->data);
+error_free_bindata:
+	GETDNS_FREE(dict->mf, newbindata);
+	return GETDNS_RETURN_MEMORY_ERROR;
+}				/* getdns_dict_util_set_dict */
+
 /*---------------------------------------- getdns_dict_set_int */
 getdns_return_t
 getdns_dict_set_int(struct getdns_dict * dict, const char *name,
@@ -499,7 +521,7 @@ getdns_indent(size_t indent)
 }				/* getdns_indent */
 
 static int
-priv_getdns_bindata_is_dname(getdns_bindata *bindata)
+_getdns_bindata_is_dname(getdns_bindata *bindata)
 {
 	size_t i = 0, n_labels = 0;
 
@@ -563,7 +585,7 @@ getdns_pp_bindata(gldns_buffer *buf, size_t indent,
 		if (gldns_buffer_printf(buf, json ? "\".\"" : "for .>") < 0)
 			return -1;
 
-	} else if (priv_getdns_bindata_is_dname(bindata)) {
+	} else if (_getdns_bindata_is_dname(bindata)) {
 		(void)gldns_wire2str_dname_buf(
 		    bindata->data, bindata->size, spc, sizeof(spc));
 		if (gldns_buffer_printf(
@@ -659,7 +681,7 @@ getdns_pp_list(gldns_buffer *buf, size_t indent, const getdns_list *list,
 				return -1;
 			if (!json && for_literals &&
 			    (strval =
-			     priv_getdns_get_const_info(int_item)->name)) {
+			     _getdns_get_const_info(int_item)->name)) {
 				if (gldns_buffer_printf(buf, "%s", strval) < 0)
 					return -1;
 			} else if (0 > 
@@ -710,7 +732,7 @@ getdns_pp_list(gldns_buffer *buf, size_t indent, const getdns_list *list,
 }				/* getdns_pp_list */
 
 static int
-priv_getdns_print_class(gldns_buffer *buf, uint32_t klass)
+_getdns_print_class(gldns_buffer *buf, uint32_t klass)
 {
 	switch (klass) {
 	case GETDNS_RRCLASS_IN:
@@ -733,7 +755,7 @@ priv_getdns_print_class(gldns_buffer *buf, uint32_t klass)
 }
 
 static int
-priv_getdns_print_opcode(gldns_buffer *buf, uint32_t opcode)
+_getdns_print_opcode(gldns_buffer *buf, uint32_t opcode)
 {
 	switch (opcode) {
 	case GETDNS_OPCODE_QUERY:
@@ -756,7 +778,7 @@ priv_getdns_print_opcode(gldns_buffer *buf, uint32_t opcode)
 }
 
 static int
-priv_getdns_print_rcode(gldns_buffer *buf, uint32_t rcode)
+_getdns_print_rcode(gldns_buffer *buf, uint32_t rcode)
 {
 	static const char *rcodes[] = {
 		" GETDNS_RCODE_NOERROR" , " GETDNS_RCODE_FORMERR" ,
@@ -807,7 +829,7 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 	i = 0;
 	indent += 2;
 	RBTREE_FOR(item, struct getdns_dict_item *,
-		(getdns_rbtree_t *)&(dict->root)) {
+		(_getdns_rbtree_t *)&(dict->root)) {
 
 		if (i && gldns_buffer_printf(buf, ",") < 0)
 			return -1;
@@ -824,7 +846,7 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 			    (strcmp(item->node.key, "type") == 0  ||
 			     strcmp(item->node.key, "type_covered") == 0 ||
 			     strcmp(item->node.key, "qtype") == 0) &&
-			    (strval = priv_getdns_rr_type_name(item->data.n))) {
+			    (strval = _getdns_rr_type_name(item->data.n))) {
 				if (gldns_buffer_printf(
 					buf, " GETDNS_RRTYPE_%s", strval) < 0)
 					return -1;
@@ -838,7 +860,7 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 			     strcmp(item->node.key, "follow_redirects") == 0 ||
 			     strcmp(item->node.key, "resolution_type") == 0) &&
 			    (strval =
-			     priv_getdns_get_const_info(item->data.n)->name)) {
+			     _getdns_get_const_info(item->data.n)->name)) {
 				if (gldns_buffer_printf(buf, " %s", strval) < 0)
 					return -1;
 				break;
@@ -846,13 +868,13 @@ getdns_pp_dict(gldns_buffer * buf, size_t indent,
 			if (!json &&
 			    (strcmp(item->node.key, "class")  == 0  ||
 			     strcmp(item->node.key, "qclass") == 0) &&
-			    priv_getdns_print_class(buf, item->data.n))
+			    _getdns_print_class(buf, item->data.n))
 				break;
 			if (!json && strcmp(item->node.key, "opcode") == 0 &&
-			    priv_getdns_print_opcode(buf, item->data.n))
+			    _getdns_print_opcode(buf, item->data.n))
 				break;
 			if (!json && strcmp(item->node.key, "rcode") == 0 &&
-			    priv_getdns_print_rcode(buf, item->data.n))
+			    _getdns_print_rcode(buf, item->data.n))
 				break;
 			if (gldns_buffer_printf(
 			    buf,(json < 2 ? " %d" : "%d"), item->data.n) < 0)
@@ -1089,7 +1111,7 @@ getdns_dict_remove_name(struct getdns_dict *this_dict, const char *name)
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 
 	/* cleanup */
-	getdns_rbtree_delete(&this_dict->root, name);
+	_getdns_rbtree_delete(&this_dict->root, name);
 	getdns_dict_item_free(&item->node, this_dict);
 
 	return GETDNS_RETURN_GOOD;
