@@ -42,29 +42,29 @@
 
 /* private inner helper used by sync and async */
 
-void priv_getdns_call_user_callback(getdns_dns_req *, getdns_dict *);
-void priv_getdns_check_dns_req_complete(getdns_dns_req *dns_req);
+void _getdns_call_user_callback(getdns_dns_req *, getdns_dict *);
+void _getdns_check_dns_req_complete(getdns_dns_req *dns_req);
 
 getdns_return_t
-priv_getdns_general_loop(getdns_context *context, getdns_eventloop *loop,
+_getdns_general_loop(getdns_context *context, getdns_eventloop *loop,
     const char *name, uint16_t request_type, getdns_dict *extensions,
     void *userarg, getdns_dns_req **dnsreq,
     getdns_callback_t callbackfn, internal_cb_t internal_cb);
 
 getdns_return_t
-priv_getdns_address_loop(getdns_context *context, getdns_eventloop *loop,
+_getdns_address_loop(getdns_context *context, getdns_eventloop *loop,
     const char *name, getdns_dict *extensions,
     void *userarg, getdns_transaction_t *transaction_id,
     getdns_callback_t callbackfn);
 
 getdns_return_t
-priv_getdns_hostname_loop(getdns_context *context, getdns_eventloop *loop,
+_getdns_hostname_loop(getdns_context *context, getdns_eventloop *loop,
     getdns_dict *address, getdns_dict *extensions,
     void *userarg, getdns_transaction_t *transaction_id,
     getdns_callback_t callbackfn);
 
 getdns_return_t
-priv_getdns_service_loop(getdns_context *context, getdns_eventloop *loop,
+_getdns_service_loop(getdns_context *context, getdns_eventloop *loop,
     const char *name, getdns_dict *extensions,
     void *userarg, getdns_transaction_t *transaction_id,
     getdns_callback_t callbackfn);
